// automatically generated by the FlatBuffers compiler, do not modify
export { Buffer } from './buffer.mjs';
export { Tensor } from './tensor.mjs';
export { TensorDim } from './tensor-dim.mjs';
export { Type, unionToType, unionListToType } from './type.mjs';

//# sourceMappingURL=Tensor_generated.mjs.map
