import { FixedSizeBinary } from '../type.js';
import { FixedWidthBuilder } from '../builder.js';
/** @ignore */
export declare class FixedSizeBinaryBuilder<TNull = any> extends FixedWidthBuilder<FixedSizeBinary, TNull> {
}
