import { Null } from '../type.js';
import { Builder } from '../builder.js';
/** @ignore */
export declare class NullBuilder<TNull = any> extends Builder<Null, TNull> {
    setValue(index: number, value: null): void;
    setValid(index: number, valid: boolean): boolean;
}
