import { Builder } from '../builder.js';
import { DataType, FixedSizeList } from '../type.js';
/** @ignore */
export declare class FixedSizeListBuilder<T extends DataType = any, TNull = any> extends Builder<FixedSizeList<T>, TNull> {
    setValue(index: number, value: T['TValue']): void;
    addChild(child: Builder<T>, name?: string): number;
}
